from django.db import models
from django.utils.safestring import mark_safe
from edc_constants.choices import POS_NEG, YES_NO
from edc_model.models import BaseUuidModel
from edc_model.validators import date_is_not_now, date_is_past

from ..model_mixins import CrfModelMixin


class HepatitisTest(CrfModelMixin, BaseUuidModel):
    # Hepatitis B Surface Antigen Test
    hbsag_performed = models.CharField(
        verbose_name="Was Hepatitis B Surface Antigen test performed?",
        max_length=15,
        choices=YES_NO,
        help_text="Answer `YES` if `ever` performed and a result is available.",
    )

    hbsag = models.CharField(
        verbose_name=mark_safe("<u>HbSAg</u>"),  # nosec B308
        max_length=15,
        choices=POS_NEG,
        null=True,
        blank=True,
    )

    hbsag_date = models.DateField(
        verbose_name=mark_safe("<i>HbSAg date</i>"),  # nosec B308
        validators=[date_is_past, date_is_not_now],
        null=True,
        blank=True,
        help_text="Approximate if not known",
    )

    hcv_performed = models.CharField(
        verbose_name="Was the patient tested for Hepatitis C?",
        max_length=15,
        choices=YES_NO,
        help_text="Answer `YES` if `ever` performed and a result is available.",
    )

    hcv = models.CharField(
        verbose_name=mark_safe("<u>HCV</u>"),  # nosec B308
        max_length=15,
        choices=POS_NEG,
        null=True,
        blank=True,
    )

    hcv_date = models.DateField(
        verbose_name=mark_safe("<i>HCV date</i>"),  # nosec B308
        validators=[date_is_past, date_is_not_now],
        null=True,
        blank=True,
        help_text="Approximate if not known",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Hepatitis Tests"
        verbose_name_plural = "Hepatitis Tests"



from .autocomplete_admin import ArvRegimensAdmin
from .birth_outcome_admin import BirthOutcomesAdmin
from .blood_results import (
    BloodResultsFbcAdmin,
    BloodResultsLftAdmin,
    BloodResultsRftAdmin,
)
from .complications_admin import ComplicationsAdmin
from .complications_glycemia_admin import ComplicationsGlycemiaAdmin
from .concomitant_medication_admin import ConcomitantMedicationAdmin
from .delivery_admin import DeliveryAdmin
from .diabetes import DmEndpointAdmin, DmFollowupAdmin
from .egfr_drop_notification_admin import EgfrDropNotificationAdmin
from .eq5d3l_admin import Eq5d3lAdmin
from .followup_examination_admin import FollowupExaminationAdmin
from .followup_vitals_admin import FollowupVitalsAdmin
from .glucose_admin import GlucoseAdmin
from .glucose_fbg_admin import GlucoseFbgAdmin
from .health_economics import HealthEconomicsSimpleAdmin, HealthEconomicsUpdateAdmin
from .hepatitis_test_admin import HepatitisTestAdmin
from .malaria_test_admin import MalariaTestAdmin
from .medication_adherence_admin import MedicationAdherenceAdmin
from .mnsi_admin import MnsiAdmin
from .other_arv_regimens_admin import OtherArvRegimensAdmin
from .patient_history_admin import PatientHistoryAdmin
from .physical_exam_admin import PhysicalExamAdmin
from .pregnancy_update_admin import PregnancyUpdateAdmin
from .sf12_admin import Sf12Admin
from .study_medication_admin import StudyMedicationAdmin
from .subject_requisition_admin import SubjectRequisitionAdmin
from .subject_visit_admin import SubjectVisitAdmin
from .subject_visit_missed_admin import SubjectVisitMissedAdmin
from .urine_dipstick_test_admin import UrineDipstickTestAdmin
from .urine_pregnancy_admin import UrinePregnancyAdmin



PHASE_TWO = 2
PHASE_THREE = 3
PHASE_THREE_ONLY = "PHASE_THREE_ONLY"


class InvalidMetaVersion(Exception):
    pass


def get_meta_version():
    from django.conf import settings

    return settings.META_PHASE



mysql_view = """ # noqa
    with appointments as (
        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,
        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,
        case when appt_timing='missed' then 1 else 0 end as `missed`
        from edc_appointment_appointment
        where visit_code_sequence=0 and appt_datetime<=now()
        order by subject_identifier, appt_datetime
        )
    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedthreeinrow2' as `report_model` from (
        select  distinct subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,
            datediff(third_date, first_date) as `interval_days`,
            datediff(now(), first_date) as `from_now_days`,
            `first_status`, `second_status`, `third_status`, sum(missed) as `missed_count`
        from (
            select subject_identifier,site_id,appt_datetime, missed,
            FIRST_VALUE(appt_status) OVER w as `third_status`,
            NTH_VALUE(appt_status, 2) OVER w as `second_status`,
            NTH_VALUE(appt_status, 3) OVER w as `first_status`,
            FIRST_VALUE(visit_code) OVER w as `third_value`,
            NTH_VALUE(visit_code, 2) OVER w as `second_value`,
            NTH_VALUE(visit_code, 3) OVER w as `first_value`,
            FIRST_VALUE(appt_datetime) OVER w as `third_date`,
            NTH_VALUE(appt_datetime, 2) OVER w as `second_date`,
            NTH_VALUE(appt_datetime, 3) OVER w as `first_date`
            from appointments
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)
        ) as A
        where `second_value` is not null and `third_value` is not null
          and `first_status`='New'
          and `second_status`='New'
          and `third_status`='New'
        group by subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,
                 datediff(`third_date`, `first_date`),
                 datediff(now(), `first_date`),
                 `first_status`, `second_status`, `third_status`
        order by subject_identifier, site_id
    ) as B
"""

pg_view = """ # noqa
    with appointments as (
        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,
        case when appt_timing='missed' then 'New' else appt_status end as appt_status,
        case when appt_timing='missed' then 1 else 0 end as missed
        from edc_appointment_appointment
        where visit_code_sequence=0 and appt_datetime<=now()
        order by subject_identifier, appt_datetime
        )
    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow2' as report_model
    from (
        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,
            EXTRACT(DAY FROM third_date - first_date) as interval_days,
            EXTRACT(DAY FROM now() - first_date) as from_now_days,
            first_status, second_status, third_status, sum(missed) as missed_count
        from (
            select subject_identifier,site_id,appt_datetime, missed,
            FIRST_VALUE(appt_status) OVER w as third_status,
            NTH_VALUE(appt_status, 2) OVER w as second_status,
            NTH_VALUE(appt_status, 3) OVER w as first_status,
            FIRST_VALUE(visit_code) OVER w as third_value,
            NTH_VALUE(visit_code, 2) OVER w as second_value,
            NTH_VALUE(visit_code, 3) OVER w as first_value,
            FIRST_VALUE(appt_datetime) OVER w as third_date,
            NTH_VALUE(appt_datetime, 2) OVER w as second_date,
            NTH_VALUE(appt_datetime, 3) OVER w as first_date
            from appointments
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)
        ) as A
        where second_value is not null and third_value is not null
          and first_status='New'
          and second_status='New'
          and third_status='New'
        group by subject_identifier, site_id,  first_value, second_value, third_value,
                 EXTRACT(DAY FROM third_date - first_date),
                 EXTRACT(DAY FROM now() - first_date),
                 first_status, second_status, third_status
        order by subject_identifier, site_id
    ) as B
"""

sqlite3_view = """ # noqa
    with appointments as (
        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,
        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,
        case when appt_timing='missed' then 1 else 0 end as `missed`
        from edc_appointment_appointment
        where visit_code_sequence=0 and appt_datetime<=now()
        order by subject_identifier, appt_datetime
        )
    SELECT *, lower(
            hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
            substr(hex( randomblob(2)), 2) || '-' ||
            substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
            substr(hex(randomblob(2)), 2) || '-' ||
            hex(randomblob(6))
          ) as id, datetime() as created, 'meta_reports.unattendedthreeinrow2' as report_model from (
        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,
            CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer) as interval_days,
            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days,
            first_status, second_status, third_status, sum(missed) as missed_count
        from (
            select subject_identifier,site_id,appt_datetime, missed,
            FIRST_VALUE(appt_status) OVER w as third_status,
            NTH_VALUE(appt_status, 2) OVER w as second_status,
            NTH_VALUE(appt_status, 3) OVER w as first_status,
            FIRST_VALUE(visit_code) OVER w as third_value,
            NTH_VALUE(visit_code, 2) OVER w as second_value,
            NTH_VALUE(visit_code, 3) OVER w as first_value,
            FIRST_VALUE(appt_datetime) OVER w as third_date,
            NTH_VALUE(appt_datetime, 2) OVER w as second_date,
            NTH_VALUE(appt_datetime, 3) OVER w as first_date
            from appointments
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)
        ) as A
        where second_value is not null and third_value is not null
          and first_status='New'
          and second_status='New'
          and third_status='New'
        group by subject_identifier, site_id,  first_value, second_value, third_value,
                 CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer),
                 CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer),
                 first_status, second_status, third_status
        order by subject_identifier, site_id
    ) as B
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



def get_blood_pressure_fields():
    return [
        "sys_blood_pressure_one",
        "dia_blood_pressure_one",
        "sys_blood_pressure_two",
        "dia_blood_pressure_two",
        "severe_htn",
    ]


def get_respiratory_o2_fields():
    return ["respiratory_rate"]



from typing import Any

from django.core.checks import messages
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from edc_subject_dashboard.views import SubjectDashboardView

from meta_reports.models import Endpoints, GlucoseSummary


class DashboardView(SubjectDashboardView):
    consent_model = "meta_consent.subjectconsentv1"
    navbar_selected_item = "consented_subject"
    visit_model = "meta_subject.subjectvisit"
    history_button_label = _("Audit")

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        try:
            Endpoints.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            pass
        else:
            url = reverse("meta_reports_admin:meta_reports_glucosesummary_changelist")
            url = f"{url}?q={self.subject_identifier}"
            message = _(
                format_html(
                    f"Subject has reached the protocol endpoint. "
                    f'See <A href="{url}">{GlucoseSummary._meta.verbose_name}</A>'
                )
            )
            self.message_user(message, level=messages.WARNING)
        return context



from .screening_part_one import ScreeningPartOneFormValidator
from .screening_part_three import ScreeningPartThreeFormValidator
from .screening_part_two import ScreeningPartTwoFormValidator
from .subject_refusal import SubjectRefusalFormValidator
from .subject_screening import SubjectScreeningFormValidator



mysql_view: str = """ # noqa
    select *, uuid() as id, now() as created,
    'meta_reports.missing_screening_ogtt_view' as report_model
      from (
        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,
        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, repeat_glucose_performed as repeated,
        p3_ltfu, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented, "" as subject_identifier,
        id as original_id
        from meta_screening_subjectscreening
        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != "Yes"
    ) as A
    order by screening_identifier
    """

pg_view: str = """ # noqa
    select *, get_random_uuid() as id, now() as created,
    'meta_reports.missing_screening_ogtt_view' as report_model
      from (
        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,
        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, repeat_glucose_performed as repeated,
        p3_ltfu, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented, "" as subject_identifier,
        id as original_id
        from meta_screening_subjectscreening
        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != "Yes"
    ) as A
    order by screening_identifier
"""

sqlite3_view = """ # noqa
SELECT *, lower(
    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
    substr(hex( randomblob(2)), 2) || '-' ||
    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
    substr(hex(randomblob(2)), 2) || '-' ||
    hex(randomblob(6))
  ) as id, datetime() as `created`,
    'meta_reports.missing_screening_ogtt_view' as report_model
      from (
        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,
        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, fbg2_value, ogtt2_value,
        repeat_glucose_performed as repeated, p3_ltfu, fbg2_datetime, ogtt2_datetime, consented, "" as subject_identifier,
        id as original_id
        from meta_screening_subjectscreening
        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != "Yes"
    ) as A
    order by screening_identifier
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



from .dm_referral_admin import DmReferralAdmin
from .end_of_study_admin import EndOfStudyAdmin
from .loss_to_followup_admin import LossToFollowupAdmin
from .off_study_medication_admin import OffStudyMedicationAdmin
from .offschedule_admin import OffScheduleAdmin
from .offschedule_dm_referral_admin import OffScheduleDmReferralAdmin
from .offschedule_postnatal_admin import OffSchedulePostnatalAdmin
from .offschedule_pregnancy_admin import OffSchedulePregnancyAdmin
from .onschedule_admin import OnScheduleAdmin
from .onschedule_dm_referral_admin import OnScheduleDmReferralAdmin
from .pregnancy_notification_admin import PregnancyNotificationAdmin
from .protocol_incident_admin import ProtocolIncidentAdmin
from .subject_transfer_admin import SubjectTransferAdmin



from django.conf import settings
from edc_auth.get_app_codenames import get_app_codenames

META_REPORTS = "META_REPORTS"
META_REPORTS_AUDIT = "META_REPORTS_AUDIT"


clinic_codenames = get_app_codenames(
    "meta_prn", "meta_subject", "meta_consent", list_app="meta_lists"
)

reports_codenames = get_app_codenames("meta_reports")

add_perms = []
if not settings.LIVE_SYSTEM:
    add_perms = [
        "meta_screening.add_screeningpartone",
        "meta_screening.add_screeningpartthree",
        "meta_screening.add_screeningparttwo",
    ]

screening_codenames = [
    *add_perms,
    "meta_screening.add_subjectrefusal",
    "meta_screening.change_screeningpartone",
    "meta_screening.change_screeningpartthree",
    "meta_screening.change_screeningparttwo",
    "meta_screening.change_subjectrefusal",
    "meta_screening.delete_screeningpartone",
    "meta_screening.delete_screeningpartthree",
    "meta_screening.delete_screeningparttwo",
    "meta_screening.delete_subjectrefusal",
    "meta_screening.view_historicalscreeningpartone",
    "meta_screening.view_historicalscreeningpartthree",
    "meta_screening.view_historicalscreeningparttwo",
    "meta_screening.view_historicalsubjectscreening",
    "meta_screening.view_historicalsubjectrefusal",
    "meta_screening.view_screeningpartone",
    "meta_screening.view_screeningpartthree",
    "meta_screening.view_screeningparttwo",
    "meta_screening.view_subjectscreening",
    "meta_screening.view_subjectrefusal",
]
screening_codenames.sort()

ae_local_reviewer = [
    "meta_subject.add_aelocalreview",
    "meta_subject.change_aelocalreview",
    "meta_subject.delete_aelocalreview",
    "meta_subject.view_aelocalreview",
    "meta_subject.view_historicalaelocalreview",
]
ae_sponsor_reviewer = [
    "meta_subject.add_aesponsorreview",
    "meta_subject.change_aesponsorreview",
    "meta_subject.delete_aesponsorreview",
    "meta_subject.view_aesponsorreview",
    "meta_subject.view_historicalaesponsorreview",
]



mysql_view = """ # noqa
    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedtwoinrow' as report_model from (
        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`,
            datediff(`second_date`, `first_date`) as `interval_days`,
            datediff(now(), `first_date`) as `from_now_days`
            from (
                select subject_identifier ,site_id, appt_datetime,
                FIRST_VALUE(`visit_code`) OVER w as `first_value`,
                NTH_VALUE(`visit_code`, 2) OVER w as `second_value`,
                NTH_VALUE(`visit_code`, 3) OVER w as `third_value`,
                FIRST_VALUE(`appt_datetime`) OVER w as `first_date`,
                NTH_VALUE(`appt_datetime`, 2) OVER w as `second_date`
                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
                and appt_datetime <= now()
                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
            ) as A1
        where `second_value` is not null and `third_value` is null
    ) as A2
    order by site_id, `from_now_days` desc
"""

pg_view = """ # noqa
    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedtwoinrow' as report_model
    from (
        select subject_identifier, site_id, appt_datetime, first_value, second_value,
            EXTRACT(DAY FROM second_date - first_date) as interval_days,
            EXTRACT(DAY FROM now() - first_date) as from_now_days
            from (
                select subject_identifier, site_id, appt_datetime,
                FIRST_VALUE(visit_code) OVER w as first_value,
                NTH_VALUE(visit_code, 2) OVER w as second_value,
                NTH_VALUE(visit_code, 3) OVER w as third_value,
                FIRST_VALUE(appt_datetime) OVER w as first_date,
                NTH_VALUE(appt_datetime, 2) OVER w as second_date
                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
                and appt_datetime <= now()
                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
            ) as A1
        where second_value is not null and third_value is null
    ) as A2
    order by site_id, from_now_days desc
"""

sqlite3_view = """  # noqa
SELECT *, lower(
    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
    substr(hex( randomblob(2)), 2) || '-' ||
    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
    substr(hex(randomblob(2)), 2) || '-' ||
    hex(randomblob(6))
  ) as id, datetime() as created, 'meta_reports.unattendedtwoinrow' as report_model from (
        select subject_identifier, site_id, appt_datetime, first_value, second_value,
            CAST(JulianDay(second_date) - JulianDay(first_date) AS Integer) as interval_days,
            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days
            from (
                select subject_identifier ,site_id, appt_datetime,
                FIRST_VALUE(visit_code) OVER w as first_value,
                NTH_VALUE(visit_code, 2) OVER w as second_value,
                NTH_VALUE(visit_code, 3) OVER w as third_value,
                FIRST_VALUE(appt_datetime) OVER w as first_date,
                NTH_VALUE(appt_datetime, 2) OVER w as second_date
                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
                and appt_datetime <= datetime()
                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
            ) as A1
        where second_value is not null and third_value is null
    ) as A2
    order by site_id, from_now_days desc
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



mysql_view = """ # noqa
    select *, uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow' as `report_model` from (
        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, `third_value`,
            datediff(third_date, first_date) as `interval_days`,
            datediff(now(), first_date) as `from_now_days`
        from (
            select subject_identifier, site_id, appt_datetime,
            FIRST_VALUE(visit_code) OVER w as `first_value`,
            NTH_VALUE(visit_code, 2) OVER w as `second_value`,
            NTH_VALUE(visit_code, 3) OVER w as `third_value`,
            FIRST_VALUE(appt_datetime) OVER w as `first_date`,
            NTH_VALUE(appt_datetime, 3) OVER w as `third_date`
            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
            and appt_datetime <= now()
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
        ) as A
    where `second_value` is not null and `third_value` is not null
    ) as B
    order by site_id, from_now_days desc
"""

pg_view = """ # noqa
    select *, gen_random_uuid() as id, now() as created,
    'meta_reports.unattendedthreeinrow' as report_model
    from (
    select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,
    EXTRACT(DAY FROM third_date - first_date) as interval_days,
    EXTRACT(DAY FROM now() - first_date) as from_now_days
    from (
    select subject_identifier,site_id,appt_datetime,
    FIRST_VALUE(visit_code) OVER w as first_value,
    NTH_VALUE(visit_code, 2) OVER w as second_value,
    NTH_VALUE(visit_code, 3) OVER w as third_value,
    FIRST_VALUE(appt_datetime) OVER w as first_date,
    NTH_VALUE(appt_datetime, 3) OVER w as third_date
    from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
    and appt_datetime <= now()
    WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
    ) as A
    where second_value is not null and third_value is not null
    ) as B
    order by site_id, from_now_days desc
"""

sqlite3_view = """ # noqa
SELECT *, lower(
    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
    substr(hex( randomblob(2)), 2) || '-' ||
    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
    substr(hex(randomblob(2)), 2) || '-' ||
    hex(randomblob(6))
  ) as id, datetime() as created,'meta_reports.unattendedthreeinrow' as report_model from (
        select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,
            CAST(JulianDay(third_date) - JulianDay(first_date) AS INTEGER) as interval_days,
            CAST(JulianDay(datetime()) - JulianDay(first_date) AS INTEGER) as from_now_days
        from (
            select subject_identifier, site_id, appt_datetime,
            FIRST_VALUE(visit_code) OVER w as first_value,
            NTH_VALUE(visit_code, 2) OVER w as second_value,
            NTH_VALUE(visit_code, 3) OVER w as third_value,
            FIRST_VALUE(appt_datetime) OVER w as first_date,
            NTH_VALUE(appt_datetime, 3) OVER w as third_date
            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'
            and appt_datetime <= datetime()
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
        ) as A
    where second_value is not null and third_value is not null
    ) as B
    order by site_id, from_now_days desc
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



from __future__ import annotations

from datetime import datetime

from django import forms
from edc_constants.constants import NO, YES
from edc_form_validators import INVALID_ERROR, FormValidator
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_utils import formatted_datetime, get_utcnow, to_utc
from edc_utils.date import to_local
from edc_utils.round_up import round_half_away_from_zero

from ..eligibility import EligibilityPartTwo


class ScreeningPartTwoFormValidator(PrnFormValidatorMixin, FormValidator):
    report_datetime_field_attr = "part_two_report_datetime"

    def clean(self):
        self.validate_report_datetimes()

        if (
            self.cleaned_data.get("agree_to_p3") == YES
            and self.cleaned_data.get("advised_to_fast") == NO
        ):
            self.raise_validation_error(
                {"advised_to_fast": "Expected YES. Patient has agreed to return"},
                INVALID_ERROR,
            )
        self.applicable_if_true(self.eligible_part_two, field_applicable="already_fasted")

        self.applicable_if(YES, NO, field="already_fasted", field_applicable="agree_to_p3")

        self.applicable_if(YES, field="agree_to_p3", field_applicable="advised_to_fast")

        self.required_if(YES, field="advised_to_fast", field_required="appt_datetime")

        self.raise_if_not_future_appt_datetime()

        if not self.part_three_report_datetime:
            self.applicable_if_true(
                self.cleaned_data.get("appt_datetime")
                and to_utc(self.cleaned_data.get("appt_datetime")) <= get_utcnow(),
                field_applicable="p3_ltfu",
                applicable_msg="Appointment date has past",
                not_applicable_msg="This field is not applicable. See appointment date above",
            )
        else:
            self.not_applicable_if_true(
                self.part_three_report_datetime is not None,
                field_applicable="p3_ltfu",
                not_applicable_msg=(
                    "The second stage of screening (P3) has already been started."
                ),
            )

        self.required_if(YES, field="p3_ltfu", field_required="p3_ltfu_date")

    @property
    def part_one_report_datetime(self) -> datetime:
        return self.instance.report_datetime

    @property
    def part_three_report_datetime(self) -> datetime | None:
        return self.instance.part_three_report_datetime

    @property
    def eligible_part_two(self) -> bool:
        """Returns False if any of the required fields is YES."""
        eligibility = EligibilityPartTwo(cleaned_data=self.cleaned_data)
        return eligibility.is_eligible

    def validate_report_datetimes(self):
        if (
            self.report_datetime
            and to_utc(self.report_datetime) < self.part_one_report_datetime
        ):
            dte = formatted_datetime(to_local(self.part_one_report_datetime))
            self.raise_validation_error(
                {
                    self.report_datetime_field_attr: (
                        "Cannot be before `Part One` report datetime. "
                        f"Expected date after {dte}. Got {self.report_datetime}"
                    )
                },
                INVALID_ERROR,
            )

        if (
            self.report_datetime
            and self.part_three_report_datetime
            and to_utc(self.report_datetime) > self.part_three_report_datetime
        ):
            dte = formatted_datetime(to_local(self.part_three_report_datetime))
            self.raise_validation_error(
                {
                    self.report_datetime_field_attr: (
                        "Cannot be after `Part Three` report datetime. "
                        f"Expected date before {dte}."
                    )
                },
                INVALID_ERROR,
            )

    def raise_if_not_future_appt_datetime(self):
        """Raises if appt_datetime is not future relative to
        part_two_report_datetime.
        """
        appt_datetime = self.cleaned_data.get("appt_datetime")
        if appt_datetime and self.report_datetime:
            tdelta = appt_datetime - to_local(self.report_datetime)

            hours = tdelta.seconds / 3600

            if (tdelta.days == 0 and hours < 10) or tdelta.days < 0:
                raise forms.ValidationError(
                    {
                        "appt_datetime": (
                            f"Invalid date. Must be at least 10hrs "
                            f"from report date/time. Got {tdelta.days} "
                            f"days {round_half_away_from_zero(hours, 1)} hrs."
                        )
                    }
                )



from edc_constants.constants import NO, TBD, YES


def calculate_inclusion_field_values_phase_two(obj):
    # BMI > 30 combined with IFG (6.1 to 6.9 mmol/L)
    if obj.calculated_bmi_value is None or not obj.converted_fbg_value:
        inclusion_a = TBD
    elif obj.calculated_bmi_value > 30.0 and 6.1 <= obj.converted_fbg_value <= 6.9:
        inclusion_a = YES
    else:
        inclusion_a = NO

    # BMI > 30 combined with OGTT (7.0 to 11.10 mmol/L)
    if obj.calculated_bmi_value is None or not obj.converted_ogtt_value:
        inclusion_b = TBD
    elif obj.calculated_bmi_value > 30.0 and 7.0 <= obj.converted_ogtt_value <= 11.10:
        inclusion_b = YES
    else:
        inclusion_b = NO

    # BMI <= 30 combined with IFG (6.3 to 6.9 mmol/L)
    if obj.calculated_bmi_value is None or not obj.converted_fbg_value:
        inclusion_c = TBD
    elif obj.calculated_bmi_value <= 30.0 and 6.3 <= obj.converted_fbg_value <= 6.9:
        inclusion_c = YES
    else:
        inclusion_c = NO

    # BMI <= 30 combined with OGTT (9.0 to 11.10 mmol/L)
    if obj.calculated_bmi_value is None or not obj.converted_ogtt_value:
        inclusion_d = TBD
    elif obj.calculated_bmi_value <= 30.0 and 9.0 <= obj.converted_ogtt_value <= 11.10:
        inclusion_d = YES
    else:
        inclusion_d = NO
    return inclusion_a, inclusion_b, inclusion_c, inclusion_d


def calculate_inclusion_field_values_phase_three(obj):
    converted_ogtt_value = obj.converted_ogtt2_value or obj.converted_ogtt_value
    # IFG (6.1 to 6.9 mmol/L)
    if not obj.converted_fbg_value:
        inclusion_a = TBD
    elif 6.1 <= obj.converted_fbg_value <= 6.9:
        inclusion_a = YES
    else:
        inclusion_a = NO

    # OGTT (7.8 to 11.10 mmol/L)
    if not converted_ogtt_value:
        inclusion_b = TBD
    elif 7.8 <= converted_ogtt_value <= 11.10:
        inclusion_b = YES
    else:
        inclusion_b = NO

    return inclusion_a, inclusion_b



# Generated by Django 4.2.11 on 2024-05-16 00:15
from django.db import migrations
from edc_model.utils import duration_hm_to_timedelta
from tqdm import tqdm


def update_duration(apps, schema_editor):
    for model in [
        "meta_subject.glucosefbg",
        "meta_subject.glucose",
        "meta_subject.bloodresultsins",
    ]:
        model_cls = apps.get_model(model)
        qs = model_cls.objects.all()
        total = qs.count()
        for obj in tqdm(qs, total=total):
            if obj.fasting_duration_str:
                tdelta = duration_hm_to_timedelta(obj.fasting_duration_str)
                obj.fasting_duration_delta = tdelta
                obj.save()


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_subject",
            "0201_alter_bloodresultsins_fasting_duration_delta_and_more",
        ),
    ]

    operations = [migrations.RunPython(update_duration)]



# Generated by Django 4.2.11 on 2024-05-15 23:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0194_remove_glucosefbg_assay_datetime_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="glucosefbg",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
    ]



from typing import Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from meta_subject.models import UrinePregnancy

from ..admin_site import meta_prn_admin
from ..forms import PregnancyNotificationForm
from ..models import PregnancyNotification


@admin.register(PregnancyNotification, site=meta_prn_admin)
class PregnancyNotificationAdmin(
    SiteModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = PregnancyNotificationForm

    additional_instructions = (
        "Important: A positive Urine βhCG must be entered before this form "
        f"may be completed (See `{UrinePregnancy._meta.verbose_name}`)."
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Pregnancy",
            {"fields": ("bhcg_confirmed", "unconfirmed_details", "bhcg_date")},
        ),
        (
            "Delivery",
            {"fields": ("edd",)},
        ),
        (
            "Followup",
            {"fields": ("may_contact",)},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "bhcg_confirmed": admin.VERTICAL,
        "may_contact": admin.VERTICAL,
    }

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "edd",
            "may_contact",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_display(request)
        custom_fields = ("edd", "may_contact")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)



from dateutil.relativedelta import relativedelta
from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_constants.constants import FEMALE, NO, YES
from edc_form_validators import INVALID_ERROR
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_registration.models import RegisteredSubject
from edc_sites.forms import SiteModelFormMixin
from edc_utils.date import to_local
from edc_visit_schedule.constants import DAY1

from meta_subject.models import UrinePregnancy

from ..models import PregnancyNotification


class PregnancyNotificationFormValidator(PrnFormValidatorMixin, FormValidator):
    def clean(self):
        try:
            RegisteredSubject.objects.get(
                subject_identifier=self.subject_identifier, gender=FEMALE
            )
        except ObjectDoesNotExist:
            self.raise_validation_error("Participant is not female.", INVALID_ERROR)

        self.required_if(NO, field="bhcg_confirmed", field_required="unconfirmed_details")
        self.required_if(YES, field="bhcg_confirmed", field_required="bhcg_date")
        self.validate_bhcg()
        self.validate_edd()

    def validate_bhcg(self):
        if (
            self.cleaned_data.get("bhcg_date")
            and self.report_datetime
            and self.cleaned_data.get("bhcg_date") > to_local(self.report_datetime).date()
        ):
            self.raise_validation_error(
                {"bhcg_date": "Expected a date on or before the report date/time."},
                INVALID_ERROR,
            )
        if (
            self.instance.id is None
            and self.cleaned_data.get("bhcg_confirmed") == YES
            and self.cleaned_data.get("bhcg_date")
        ):
            if (
                not UrinePregnancy.objects.filter(
                    subject_visit__subject_identifier=self.subject_identifier,
                    notified=False,
                    assay_date=self.cleaned_data.get("bhcg_date"),
                )
                .exclude(subject_visit__visit_code=DAY1, subject_visit__visit_code_sequence=0)
                .exists()
            ):
                self.raise_validation_error(
                    "Invalid. A positive Urine βhCG cannot be found. "
                    "Ensure the UPT has been entered, the date matches, and the "
                    "UPT is not a baseline UPT. "
                    f"See also PRN CRF {UrinePregnancy._meta.verbose_name}",
                    INVALID_ERROR,
                )

    def validate_edd(self):
        if (
            self.cleaned_data.get("edd")
            and self.cleaned_data.get("bhcg_date")
            and self.cleaned_data.get("edd") < self.cleaned_data.get("bhcg_date")
        ):
            self.raise_validation_error(
                {"edd": "Expected a date after the UPT date."}, INVALID_ERROR
            )
        if (
            self.cleaned_data.get("edd")
            and self.cleaned_data.get("bhcg_date")
            and self.cleaned_data.get("edd")
            > self.cleaned_data.get("bhcg_date") + relativedelta(months=9)
        ):
            self.raise_validation_error(
                {"edd": "Expected a date within 9 months of UPT date."}, INVALID_ERROR
            )
        if (
            self.report_datetime
            and not self.cleaned_data.get("bhcg_date")
            and self.cleaned_data.get("bhcg_confirmed") == NO
        ):
            if (
                self.report_datetime
                and self.cleaned_data.get("edd") < to_local(self.report_datetime).date()
            ):
                self.raise_validation_error(
                    {"edd": "Expected a date after the report date."}, INVALID_ERROR
                )
            if self.report_datetime and self.cleaned_data.get("edd") > to_local(
                self.report_datetime
            ).date() + relativedelta(months=9):
                self.raise_validation_error(
                    {"edd": "Expected a date within 9 months of report date."}, INVALID_ERROR
                )


class PregnancyNotificationForm(
    SiteModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    ActionItemFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = PregnancyNotificationFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = PregnancyNotification
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-07-22 19:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0133_auto_20220722_2140"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsrft",
            name="old_egfr_drop_value",
            field=models.DecimalField(
                decimal_places=4,
                editable=False,
                help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
                max_digits=10,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="old_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                editable=False,
                help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
                max_digits=8,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="old_egfr_drop_value",
            field=models.DecimalField(
                decimal_places=4,
                editable=False,
                help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
                max_digits=10,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="old_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                editable=False,
                help_text="incorrect ckd-epi calculation (w/ 1.150 as ethnicity factor)",
                max_digits=8,
                null=True,
            ),
        ),
    ]



from .endpoints_admin import EndpointsAdmin
from .endpoints_all_admin import EndpointsAllAdmin
from .unmanaged import (
    GlucoseSummaryAdmin,
    MissingScreeningOgttAdmin,
    PatientHistoryMissingBaselineCd4Admin,
    UnattendedThreeInRow2Admin,
    UnattendedThreeInRowAdmin,
    UnattendedTwoInRowAdmin,
)



import pandas as pd
from edc_analytics.constants import N_ONLY, N_WITH_COL_PROP, N_WITH_ROW_PROP
from edc_analytics.row import RowDefinition, RowDefinitions
from edc_analytics.table import Table


class EligibleP12Table(Table):
    def __init__(self, main_df: pd.DataFrame = None):
        super().__init__(colname="", main_df=main_df, title="Eligible by Part 1/2")

    @property
    def row_definitions(self) -> RowDefinitions:
        df_tmp = self.main_df.copy()
        row_defs = RowDefinitions(reverse_rows=False)
        row0 = RowDefinition(
            title=self.title,
            label=self.default_sublabel,
            condition=(df_tmp["gender"].notna()),
            columns={
                "F": (N_ONLY, 2),
                "M": (N_ONLY, 2),
                "All": (N_ONLY, 2),
            },
            drop=False,
        )
        row_defs.add(row0)
        columns = {
            "F": (N_WITH_COL_PROP, 2),
            "M": (N_WITH_COL_PROP, 2),
            "All": (N_WITH_ROW_PROP, 2),
        }
        row_defs.add(
            RowDefinition(
                label="Eligible P1",
                condition=(df_tmp["eligible_part_one"] == "Yes"),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                label="Eligible P2",
                condition=(df_tmp["eligible_part_two"] == "Yes"),
                columns=columns,
                drop=False,
            )
        )

        row_defs.add(
            RowDefinition(
                label="Eligible P1 & P2",
                condition=(df_tmp["eligible_part_one"] == "Yes")
                & (df_tmp["eligible_part_two"] == "Yes"),
                columns=columns,
                drop=False,
            )
        )
        # note already_fasted accepts Y/N/NA
        row_defs.add(
            RowDefinition(
                label="Glucose tested (fasted on day)",
                condition=(
                    (df_tmp["already_fasted"] == "Yes")
                    & ((df_tmp["fbg"].notna()) | (df_tmp["ogtt"].notna()))
                ),
                columns=columns,
                drop=True,
            )
        )

        row_defs.add(
            RowDefinition(
                label="Glucose tested (returned fasted)",
                condition=(
                    (df_tmp["already_fasted"] == "No")
                    & ((df_tmp["fbg"].notna()) | (df_tmp["ogtt"].notna()))
                ),
                columns=columns,
                drop=True,
            )
        )

        row_defs.add(
            RowDefinition(
                label="Glucose tested (N/A)",
                condition=(df_tmp["already_fasted"] == "N/A")
                & ((df_tmp["fbg"].notna()) | (df_tmp["ogtt"].notna())),
                columns=columns,
                drop=True,
            )
        )

        row_defs.add(
            RowDefinition(
                label="No glucose test (not fasted on day and did not return)",
                condition=(
                    (df_tmp["eligible_part_one"] == "Yes")
                    & (df_tmp["eligible_part_two"] == "Yes")
                    & (df_tmp["fbg"].isna())
                    & (df_tmp["ogtt"].isna())
                ),
                columns=columns,
                drop=False,
            )
        )
        return row_defs



from typing import Type

from django.conf import settings
from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.db.models import QuerySet
from django.template.loader import render_to_string
from django.urls import reverse
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import (
    ModelAdminFormInstructionsMixin,
    TemplatesModelAdminMixin,
)
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_sites.admin.list_filters import SiteListFilter
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ..tasks import update_endpoints_table


def update_endpoints_table_action(modeladmin, request, queryset):
    subject_identifiers = []
    if queryset.count() != modeladmin.model.objects.count():
        subject_identifiers = [o.subject_identifier for o in queryset]
    if settings.CELERY_ENABLED:
        return update_endpoints_table.delay(subject_identifiers)
    return update_endpoints_table(subject_identifiers)


update_endpoints_table_action.short_description = "Regenerate report for selected subjects"


class EndpointsModelAdminMixin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    ModelAdminFormInstructionsMixin,
    TemplatesModelAdminMixin,
):
    queryset_filter: dict | None = None
    actions = [update_endpoints_table_action]
    qa_report_list_display_insert_pos = 2
    ordering = ["-fbg_datetime"]
    list_display = [
        "dashboard",
        "subject",
        "visit",
        "fbg_date",
        "fast",
        "fbg",
        "ogtt",
        "endpoint",
        "last_updated",
        "offstudy_datetime",
        "offstudy_reason",
    ]

    list_filter = [
        "endpoint_label",
        ScheduleStatusListFilter,
        SiteListFilter,
    ]

    search_fields = ["subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        qs = super().get_queryset(request)
        if self.queryset_filter:
            qs = qs.filter(**self.queryset_filter)
        return qs

    def get_list_filter(self, request) -> tuple[str | Type[SimpleListFilter], ...]:
        list_filter = super().get_list_filter(request)
        list_filter = list_filter + (SiteListFilter,)
        return list_filter

    @admin.display(description="subject", ordering="subject_identifier")
    def subject(self, obj=None):
        return obj.subject_identifier

    @admin.display(description="visit", ordering="visit_code")
    def visit(self, obj=None):
        return obj.visit_code

    @admin.display(description="FBG DATE", ordering="fbg_datetime")
    def fbg_date(self, obj=None):
        return obj.fbg_datetime.date() if obj.fbg_datetime else None

    @admin.display(description="FAST", ordering="fasting")
    def fast(self, obj=None):
        return obj.fasting

    @admin.display(description="FBG", ordering="fbg_value")
    def fbg(self, obj=None):
        return obj.fbg_value

    @admin.display(description="OGTT", ordering="ogtt_value")
    def ogtt(self, obj=None):
        return obj.ogtt_value

    @admin.display(description="endpoint", ordering="endpoint_label")
    def endpoint(self, obj=None):
        url = reverse("meta_reports_admin:meta_reports_glucosesummary_changelist")
        return render_to_string(
            "meta_reports/columns/subject_identifier_column.html",
            {
                "subject_identifier": obj.subject_identifier,
                "url": url,
                "label": obj.endpoint_label,
            },
        )

    @admin.display(description="last_updated", ordering="created")
    def last_updated(self, obj=None):
        return obj.created



mysql_view: str = """ # noqa
select *, uuid() as id, now() as created, 'meta_reports.glucose_summary_view' as report_model
from (
    select v.subject_identifier, fbg_value, fbg_datetime, null as 'ogtt_value', null as 'ogtt_datetime',
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as 'fasted',
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucosefbg as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    UNION
    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as 'fasted',
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucose as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
) as A
order by subject_identifier, fbg_datetime;
"""

pg_view: str = """ # noqa
select *, get_random_uuid() as id, now() as created, 'meta_reports.glucose_summary_view' as report_model
from (
    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucosefbg as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    UNION
    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucose as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
) as A
order by subject_identifier, fbg_datetime;
"""

sqlite3_view: str = """ # noqa
SELECT *, lower(
    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
    substr(hex( randomblob(2)), 2) || '-' ||
    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
    substr(hex(randomblob(2)), 2) || '-' ||
    hex(randomblob(6))
  ) as id, datetime() as `created`, 'meta_reports.glucose_summary_view' as report_model
from (
    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucosefbg as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    UNION
    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucose as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    ) as A
order by subject_identifier, fbg_datetime;
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



# Generated by Django 3.2.13 on 2022-09-08 15:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0147_auto_20220907_1505"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="refill_end_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="Leave blank to auto-calculate if refilling to the next scheduled visit",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="refill_to_next_visit",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                help_text="If YES, leave refill end date blank to auto-calculate",
                max_length=25,
                verbose_name="Refill to the next scheduled visit",
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="refill_end_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="Leave blank to auto-calculate if refilling to the next scheduled visit",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="refill_to_next_visit",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                help_text="If YES, leave refill end date blank to auto-calculate",
                max_length=25,
                verbose_name="Refill to the next scheduled visit",
            ),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-13 01:47

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0091_auto_20210929_2324"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="haemoglobin_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Haemoglobin",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="hct_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999.0),
                ],
                verbose_name="Hematocrit",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mch_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCH",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mchc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCHC",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="mcv_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCV",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="platelets_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="Platelets",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="rbc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999999.0),
                ],
                verbose_name="Red blood cell count",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="wbc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="WBC",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="glucose_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="Blood Glucose",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="albumin_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="Serum albumin",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="ALT",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="amylase_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="Serum Amylase",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ast_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="AST",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ggt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="GGT",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="urea_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Urea (BUN)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="haemoglobin_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Haemoglobin",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="hct_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999.0),
                ],
                verbose_name="Hematocrit",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mch_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCH",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mchc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCHC",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="mcv_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="MCV",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="platelets_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="Platelets",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="rbc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999999.0),
                ],
                verbose_name="Red blood cell count",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="wbc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="WBC",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="glucose_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="Blood Glucose",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="albumin_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="Serum albumin",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="ALT",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="amylase_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.9),
                ],
                verbose_name="Serum Amylase",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ast_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="AST",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ggt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="GGT",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="urea_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[django.core.validators.MinValueValidator(0.0)],
                verbose_name="Urea (BUN)",
            ),
        ),
    ]



from django.contrib import admin, messages
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.models import Medication
from edc_pharmacy.prescribe import create_prescription

from meta_consent.models import SubjectConsent
from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN


@admin.action(permissions=["view"], description="Create missing METFORMIN prescription")
def create_missing_metformin_rx(modeladmin, request, queryset):
    medication = Medication.objects.get(name=METFORMIN)
    total = queryset.count()
    subject_identifiers = queryset.values_list("subject_identifier", flat=True)
    subject_consents_wo_rx = SubjectConsent.objects.filter(
        subject_identifier__in=subject_identifiers
    )
    created = 0
    exist = 0
    for instance in subject_consents_wo_rx:
        try:
            create_prescription(
                subject_identifier=instance.subject_identifier,
                report_datetime=instance.consent_datetime,
                randomizer_name=get_meta_version(),
                medications=[medication],
                site_id=instance.site.id,
            )
        except PrescriptionAlreadyExists:
            exist += 1
        else:
            created += 1
    messages.success(
        request,
        f"Created {created}/{total} missing {medication.display_name} prescriptions. "
        f"Got {exist}/{total} prescriptions already exist",
    )



# Generated by Django 4.1.2 on 2023-03-01 18:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_ae", "0017_auto_20221130_2257"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deathreporttmg",
            name="cause_of_death_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If No, explain in the narrative below",
                max_length=15,
                null=True,
                verbose_name="Is the cause of death agreed between study doctor and TMG member?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmg",
            name="cause_of_death_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If No, explain in the narrative below",
                max_length=15,
                null=True,
                verbose_name="Is the cause of death agreed between study doctor and TMG member?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmgsecond",
            name="cause_of_death_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If No, explain in the narrative below",
                max_length=15,
                null=True,
                verbose_name="Is the cause of death agreed between study doctor and TMG member?",
            ),
        ),
    ]



# Generated by Django 4.2.10 on 2024-03-07 23:06

import django.contrib.sites.managers
from django.db import migrations, models
import edc_consent.managers


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0022_alter_historicalsubjectconsent_site_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="SubjectConsentV1",
            fields=[],
            options={
                "verbose_name": "Consent V1",
                "verbose_name_plural": "Consent V1",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("meta_consent.subjectconsent",),
            managers=[
                ("objects", edc_consent.managers.ConsentObjectsByCdefManager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.RemoveConstraint(
            model_name="subjectconsent",
            name="meta_consent_subjectconsent_version_uniq",
        ),
        migrations.RenameField(
            model_name="historicalsubjectconsent",
            old_name="updates_versions",
            new_name="update_versions",
        ),
        migrations.RenameField(
            model_name="subjectconsent",
            old_name="updates_versions",
            new_name="update_versions",
        ),
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="consent_definition_name",
            field=models.CharField(
                editable=False, max_length=50, null=True, verbose_name="Consent definition"
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="consent_definition_name",
            field=models.CharField(
                editable=False, max_length=50, null=True, verbose_name="Consent definition"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="version",
            field=models.CharField(
                editable=False,
                help_text="See 'consent definition' for consent versions by period.",
                max_length=10,
                verbose_name="Consent version",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="version",
            field=models.CharField(
                editable=False,
                help_text="See 'consent definition' for consent versions by period.",
                max_length=10,
                verbose_name="Consent version",
            ),
        ),
        migrations.AddConstraint(
            model_name="subjectconsent",
            constraint=models.UniqueConstraint(
                fields=("version", "screening_identifier", "subject_identifier"),
                name="meta_consent_subjectconsent_version_uniq",
            ),
        ),
    ]



# Generated by Django 3.0.4 on 2020-04-20 15:16

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0029_auto_20200420_1751"),
    ]

    operations = [
        migrations.AlterField(
            model_name="coronakap",
            name="diabetic_dx_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first learn you had <u>diabetes</u>?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="hiv_pos_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first test positive?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="hypertensive_dx_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first learn you had <u>hypertension</u>?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="shared_housing_one",
            field=models.IntegerField(
                help_text="Family / people who typically spend more than 14 nights per month in your home.",
                verbose_name="How many people live together in your home / dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="coronakap",
            name="shared_housing_two",
            field=models.IntegerField(
                help_text="Visitors, friends, relatives, etc",
                verbose_name="In a typical month, how many <u>additional people</u> spend more than one night in your home / dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="diabetic_dx_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first learn you had <u>diabetes</u>?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="hiv_pos_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first test positive?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="hypertensive_dx_year",
            field=models.IntegerField(
                blank=True,
                help_text="format YYYY",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1950),
                    django.core.validators.MaxValueValidator(2020),
                ],
                verbose_name="If 'Yes', what year did you first learn you had <u>hypertension</u>?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="shared_housing_one",
            field=models.IntegerField(
                help_text="Family / people who typically spend more than 14 nights per month in your home.",
                verbose_name="How many people live together in your home / dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcoronakap",
            name="shared_housing_two",
            field=models.IntegerField(
                help_text="Visitors, friends, relatives, etc",
                verbose_name="In a typical month, how many <u>additional people</u> spend more than one night in your home / dwelling?",
            ),
        ),
    ]



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import OffScheduleModelFormMixin

from ..models import OffScheduleDmReferral


class OffScheduleDmReferralFormValidator(PrnFormValidatorMixin, FormValidator):
    pass


class OffScheduleDmReferralForm(
    OffScheduleModelFormMixin,
    SiteModelFormMixin,
    ActionItemFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = OffScheduleDmReferralFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = OffScheduleDmReferral
        fields = "__all__"



from django.apps import AppConfig as DjangoApponfig

from meta_edc.meta_version import get_meta_version


class AppConfig(DjangoApponfig):
    name = "meta_auth"
    verbose_name = f"META{get_meta_version()}: Authentication and Permissions"
    default_auto_field = "django.db.models.BigAutoField"



# Generated by Django 4.2.5 on 2023-10-17 22:25

import _socket
from django.db import migrations, models
import django_audit_fields.fields.hostname_modification_field


class Migration(migrations.Migration):
    dependencies = [
        ("meta_consent", "0017_auto_20220929_1742"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectconsent",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "consent_datetime",
                "ordering": ("created",),
                "verbose_name": "Subject Consent",
                "verbose_name_plural": "Subject Consents",
            },
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="ethnicity",
            field=models.CharField(
                editable=False, help_text="from screening", max_length=15, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="ethnicity",
            field=models.CharField(
                editable=False, help_text="from screening", max_length=15, null=True
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-10-12 03:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0056_alter_historicalscreeningpartone_agree_to_p3_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="calculated_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mL/min/1.73m2 (system calculated)",
                max_digits=10,
                null=True,
                verbose_name="eGFR",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_creatinine_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="calculated_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mL/min/1.73m2 (system calculated)",
                max_digits=10,
                null=True,
                verbose_name="eGFR",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_creatinine_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="calculated_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mL/min/1.73m2 (system calculated)",
                max_digits=10,
                null=True,
                verbose_name="eGFR",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_creatinine_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="calculated_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mL/min/1.73m2 (system calculated)",
                max_digits=10,
                null=True,
                verbose_name="eGFR",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_creatinine_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="calculated_egfr_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mL/min/1.73m2 (system calculated)",
                max_digits=10,
                null=True,
                verbose_name="eGFR",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_creatinine_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="μmol/L (micromoles/L) (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Serum creatinine",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=10,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine level",
            ),
        ),
    ]



from dateutil.relativedelta import relativedelta
from edc_constants.constants import YES
from edc_utils import get_utcnow
from faker import Faker
from model_bakery.recipe import Recipe

from .models import DmReferral, PregnancyNotification

fake = Faker()


pregnancynotification = Recipe(
    PregnancyNotification,
    # site=None,
    action_identifier=None,
    subject_identifier=None,
    report_datetime=get_utcnow(),
    bhcg_confirmed=YES,
    unconfirmed_details=None,
    edd=get_utcnow() + relativedelta(months=6),
)

dmreferral = Recipe(
    DmReferral,
    action_identifier=None,
    subject_identifier=None,
    report_datetime=get_utcnow(),
)



from django import forms
from edc_constants.constants import NO, NONE, OTHER, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import PatientHistory
from .mixins import ArvHistoryFormValidatorMixin


class PatientHistoryFormValidator(ArvHistoryFormValidatorMixin, CrfFormValidator):
    def clean(self):
        self.m2m_single_selection_if(NONE, m2m_field="symptoms")

        self.m2m_other_specify(OTHER, m2m_field="symptoms", field_other="other_symptoms")

        self.date_not_before(
            "hiv_diagnosis_date",
            "arv_initiation_date",
            "Invalid. Cannot be before HIV diagnosis date.",
        )

        self.validate_arv_history_fields()

        self.required_if(
            YES,
            field="on_htn_treatment",
            field_required="htn_treatment",
        )

        self.m2m_other_specify(
            OTHER,
            m2m_field="htn_treatment",
            field_other="other_htn_treatment",
        )

        self.applicable_if(
            YES,
            field="dyslipidaemia_diagnosis",
            field_applicable="on_dyslipidaemia_treatment",
        )

        self.applicable_if(
            YES,
            field="on_dyslipidaemia_treatment",
            field_applicable="dyslipidaemia_rx",
        )

        self.applicable_if(NO, field="current_smoker", field_applicable="former_smoker")

        self.m2m_single_selection_if(NONE, m2m_field="past_year_symptoms")
        self.m2m_other_specify(OTHER, m2m_field="dm_symptoms", field_other="other_dm_symptoms")


class PatientHistoryForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = PatientHistoryFormValidator

    class Meta:
        model = PatientHistory
        fields = "__all__"



from django.apps import AppConfig as DjangoAppConfig
from django.db.models.signals import post_migrate
from django_extensions.management.color import color_style

from .prepare_meta_pharmacy import prepare_meta_pharmacy

style = color_style()


def post_migrate_populate_pharmacy_models(sender=None, **kwargs):  # noqa
    """Create or update pharmacy static models."""
    prepare_meta_pharmacy()


class AppConfig(DjangoAppConfig):
    name = "meta_pharmacy"
    verbose_name = "META Pharmacy"

    def ready(self):
        post_migrate.connect(post_migrate_populate_pharmacy_models, sender=self)



# Generated by Django 3.2.11 on 2022-03-10 01:39

import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0019_auto_20220309_2230"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalpregnancynotification",
            name="may_contact",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                default="Yes",
                help_text="If YES/UNSURE, a visit will be scheduled on the EDD. If NO, the participant will be taken off study now",
                max_length=15,
                verbose_name="Has the participant agreed to be contacted to provide information on the delivery and birth outcomes?",
            ),
        ),
        migrations.AddField(
            model_name="pregnancynotification",
            name="may_contact",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("not_sure", "Not sure")],
                default="Yes",
                help_text="If YES/UNSURE, a visit will be scheduled on the EDD. If NO, the participant will be taken off study now",
                max_length=15,
                verbose_name="Has the participant agreed to be contacted to provide information on the delivery and birth outcomes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="edd",
            field=models.DateField(
                validators=[edc_model.validators.date.date_is_future],
                verbose_name="Estimated date of delivery",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="edd",
            field=models.DateField(
                validators=[edc_model.validators.date.date_is_future],
                verbose_name="Estimated date of delivery",
            ),
        ),
    ]



from datetime import datetime

import numpy as np
import pandas as pd
from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_constants.constants import YES

from meta_analytics.dataframes import EndpointByDate


class TestEndpointsByDate(TestCase):
    def test_case_one(self):
        def d(m: int, w: int | None = None):
            if w:
                return datetime.now() - relativedelta(months=m, weeks=w)
            return datetime.now() - relativedelta(months=m)

        def append_to(subject_df, endpoint_df):
            if endpoint_df.empty:
                endpoint_df = subject_df.copy()
            elif subject_df.empty:
                pass
            else:
                endpoint_df = pd.concat([endpoint_df, subject_df])
                endpoint_df = self.endpoint_df.sort_values(
                    by=["subject_identifier", "visit_code"]
                )
                endpoint_df.reset_index(drop=True, inplace=True)

        subject_identifier = "101-1"
        data = {
            "subject_identifier": [subject_identifier] * 10,
            "visit_code": [
                1000.0,
                1005.0,
                1010.0,
                1030.0,
                1060.0,
                1090.0,
                1120.0,
                1115.0,
                1118.0,
                1210.0,
            ],
            "fasting": [YES] * 10,
            "fbg_datetime": [
                d(0),
                d(0, 2),
                d(1),
                d(3),
                d(6),
                d(9),
                d(12),
                d(15),
                d(18),
                d(21),
            ],
            "fbg_value": [np.nan, 6.9, 6.9, 7.1, 7.1, 6.4, 7.0, np.nan, 8.1, np.nan],
            "ogtt_value": [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                7.9,
                np.nan,
            ],
        }

        fbgs = [np.nan, 6.9, 6.9, 7.1, 7.1, 6.4, 7.0, np.nan, 8.1, np.nan]
        ogtts = [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, 7.9, np.nan]

        fbgs = [np.nan, 6.9, 6.9, 7.1, 7.1, 6.4, 7.0, 8.1, np.nan, np.nan]
        ogtts = [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, 7.9, np.nan, np.nan]

        data.update(fbg_values=fbgs, ogtt_values=ogtts)
        subject_df = pd.DataFrame(data)
        subject_df["fasting"] = subject_df.apply(
            lambda r: np.nan if pd.isna(r.fbg_value) else r.fbg_value, axis=1
        )
        subject_df["fbg_datetime"] = subject_df.apply(
            lambda r: np.nan if pd.isna(r.fbg_value) else r.fbg_datetime, axis=1
        )

        endpoint_df = pd.DataFrame()
        for index, row in subject_df.iterrows():
            cls = EndpointByDate(index, row, fbg_threshhold=7.0, ogtt_threshhold=11.1)
            append_to(cls.subject_df, endpoint_df)

        endpoint_df



# Generated by Django 5.0 on 2024-01-11 01:42

from django.db import migrations
from django.db.migrations import RunPython


def update_model_name(apps, schema_editor):
    model_cls = apps.get_model("meta_consent.subjectconsent")
    model_cls.objects.update(model_name="meta_consent.subjectconsent")


class Migration(migrations.Migration):
    dependencies = [
        ("meta_consent", "0020_historicalsubjectconsent_model_name_and_more"),
    ]

    operations = [RunPython(update_model_name)]



from .screening_part_one_admin import ScreeningPartOneAdmin
from .screening_part_three_admin import ScreeningPartThreeAdmin
from .screening_part_two_admin import ScreeningPartTwoAdmin
from .subject_refusal_admin import SubjectRefusalAdmin
from .subject_screening_admin import SubjectScreeningAdmin



from .birth_outcomes import BirthOutcomes
from .blood_results import (
    BloodResultsFbc,
    BloodResultsHba1c,
    BloodResultsIns,
    BloodResultsLft,
    BloodResultsLipid,
    BloodResultsRft,
)
from .complications import Complications
from .complications_glycemia import ComplicationsGlycemia
from .concomitant_medication import ConcomitantMedication
from .delivery import Delivery
from .diabetes import DmEndpoint, DmFollowup
from .diet_and_lifestyle import DietAndLifestyle
from .egfr_drop_notification import EgfrDropNotification
from .eq5d3l import Eq5d3l
from .followup_examination import FollowupExamination
from .followup_vitals import FollowupVitals
from .glucose import Glucose
from .glucose_fbg import GlucoseFbg
from .health_economics import (
    HealthEconomics,
    HealthEconomicsSimple,
    HealthEconomicsUpdate,
)
from .hepatitis_test import HepatitisTest
from .malaria_test import MalariaTest
from .medication_adherence import MedicationAdherence
from .mnsi import Mnsi
from .other_arv_regimens import OtherArvRegimens
from .other_arv_regimens_detail import OtherArvRegimensDetail
from .patient_history import PatientHistory
from .physical_exam import PhysicalExam
from .pregnancy_update import PregnancyUpdate
from .sf12 import Sf12
from .signals import (
    study_medication_on_pre_save,
    update_glucose_endpoints_for_subject_on_post_save,
    update_pregnancy_notification_on_delivery_post_save,
    update_schedule_on_delivery_post_save,
)
from .study_medication import StudyMedication
from .subject_requisition import SubjectRequisition
from .subject_visit import SubjectVisit
from .subject_visit_missed import SubjectVisitMissed
from .urine_dipstick_test import UrineDipstickTest
from .urine_pregnancy import UrinePregnancy



from django import forms

from ..models import AeSponsorReview
from .modelform_mixins import AeReviewModelFormMixin


class AeSponsorReviewForm(AeReviewModelFormMixin, forms.ModelForm):
    class Meta(AeReviewModelFormMixin.Meta):
        model = AeSponsorReview
        fields = "__all__"



from django import forms
from edc_constants.constants import YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_model_form.mixins import InlineModelFormMixin

from ..models import OtherArvRegimens


class OtherArvRegimensFormValidator(CrfFormValidator):
    def clean(self):
        self.required_if(
            YES,
            field="has_other_regimens",
            field_required="arv_regimen",
            field_required_inline_set="otherarvregimensdetail_set",
            required_msg="Based on your response, additional ARV Regimen(s) are required.",
            not_required_msg=(
                "Based on your response, additional ARV Regimen(s) are NOT required."
            ),
        )


class OtherArvRegimensForm(InlineModelFormMixin, CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = OtherArvRegimensFormValidator

    def clean(self):
        cleaned_data = super().clean()
        self.unique_inline_values_or_raise(
            field="arv_regimen",
            inline_model="meta_subject.otherarvregimensdetail",
            field_label="ARV Regimens",
        )
        self.unique_inline_values_or_raise(
            field="arv_regimen_start_date",
            inline_model="meta_subject.otherarvregimensdetail",
            field_label="ARV Start Dates",
        )
        self.dates_not_after_report_datetime(
            field="arv_regimen_start_date",
            inline_model="meta_subject.otherarvregimensdetail",
            field_label="ARV Start Dates",
        )
        return cleaned_data

    class Meta:
        model = OtherArvRegimens
        fields = "__all__"



# Generated by Django 3.2.11 on 2022-03-09 18:06

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.validators.date
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_action_item", "0028_auto_20210203_0706"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("meta_prn", "0017_auto_20220307_1929"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="delivery",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Delivery",
                "verbose_name_plural": "Delivery",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldelivery",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Delivery",
            },
        ),
        migrations.AlterField(
            model_name="delivery",
            name="maternal_outcome",
            field=models.CharField(
                choices=[
                    ("no_complications", "No complications"),
                    ("complications_full_recovery", "Complications with full recovery"),
                    (
                        "complications_ongoing_recovery",
                        "Complications with ongoing recovery",
                    ),
                    ("dead", "Maternal mortality"),
                ],
                max_length=50,
                verbose_name="What was the maternal outcome of the pregnancy?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="maternal_outcome",
            field=models.CharField(
                choices=[
                    ("no_complications", "No complications"),
                    ("complications_full_recovery", "Complications with full recovery"),
                    (
                        "complications_ongoing_recovery",
                        "Complications with ongoing recovery",
                    ),
                    ("dead", "Maternal mortality"),
                ],
                max_length=50,
                verbose_name="What was the maternal outcome of the pregnancy?",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="delivery",
            unique_together={("subject_identifier", "delivery_datetime")},
        ),
        migrations.CreateModel(
            name="HistoricalPregnancyNotification",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "bhcg_confirmed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Has the pregnancy been confirmed by urine βHCG?",
                    ),
                ),
                (
                    "unconfirmed_details",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="If no, please provide details",
                    ),
                ),
                (
                    "edd",
                    models.DateField(
                        validators=[edc_model.validators.date.date_is_future],
                        verbose_name="Estimated date of delivery :",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Pregnancy Notification",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalBirthOutcomes",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("maternal_identifier", models.CharField(max_length=50)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                ("birth_order", models.IntegerField()),
                (
                    "birth_outcome",
                    models.CharField(
                        choices=[
                            ("live_at_term", "Live birth at term"),
                            ("live_preterm", "Live birth preterm"),
                            ("stillbirth", "Stillbirth"),
                            ("miscarriage", "Miscarriage"),
                        ],
                        max_length=25,
                        verbose_name="Outcome",
                    ),
                ),
                (
                    "birth_weight",
                    models.IntegerField(
                        blank=True,
                        help_text="gm",
                        null=True,
                        verbose_name="Weight (gm)",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "delivery",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_prn.delivery",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Birth Outcomes",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="PregnancyNotification",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "bhcg_confirmed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Has the pregnancy been confirmed by urine βHCG?",
                    ),
                ),
                (
                    "unconfirmed_details",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="If no, please provide details",
                    ),
                ),
                (
                    "edd",
                    models.DateField(
                        validators=[edc_model.validators.date.date_is_future],
                        verbose_name="Estimated date of delivery :",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Pregnancy Notification",
                "verbose_name_plural": "Pregnancy Notifications",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "unique_together": {("subject_identifier", "edd")},
            },
        ),
        migrations.CreateModel(
            name="BirthOutcomes",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                ("maternal_identifier", models.CharField(max_length=50)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                ("birth_order", models.IntegerField()),
                (
                    "birth_outcome",
                    models.CharField(
                        choices=[
                            ("live_at_term", "Live birth at term"),
                            ("live_preterm", "Live birth preterm"),
                            ("stillbirth", "Stillbirth"),
                            ("miscarriage", "Miscarriage"),
                        ],
                        max_length=25,
                        verbose_name="Outcome",
                    ),
                ),
                (
                    "birth_weight",
                    models.IntegerField(
                        blank=True,
                        help_text="gm",
                        null=True,
                        verbose_name="Weight (gm)",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "delivery",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_prn.delivery",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Birth Outcomes",
                "verbose_name_plural": "Birth Outcomes",
                "ordering": ["maternal_identifier", "birth_order"],
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "unique_together": {("maternal_identifier", "birth_order")},
            },
        ),
    ]



from .glucose_summary_admin import GlucoseSummaryAdmin
from .missing_screening_ogtt_admin import MissingScreeningOgttAdmin
from .patient_history_missing_baseline_cd4_admin import (
    PatientHistoryMissingBaselineCd4Admin,
)
from .unattended_three_in_row2_admin import UnattendedThreeInRow2Admin
from .unattended_three_in_row_admin import UnattendedThreeInRowAdmin
from .unattended_two_in_row_admin import UnattendedTwoInRowAdmin



from copy import deepcopy
from typing import Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_screening_admin
from ..forms import (
    ScreeningPartThreeForm,
    calculated_fields,
    part_one_fields,
    part_two_fields,
)
from ..models import ScreeningPartThree
from .fieldsets import (
    calculated_values_fieldset,
    comments_fieldset,
    get_part_one_fieldset,
    get_part_three_creatinine_fieldset,
    get_part_three_glucose_fieldset,
    get_part_three_hba1c_fieldset,
    get_part_three_pregnancy_fieldset,
    get_part_three_repeat_glucose_fieldset,
    get_part_three_report_datetime_fieldset,
    get_part_three_vitals_fieldset,
    get_part_two_fieldset,
)
from .list_filters import EligibilityPending, P3ApptListFilter, P3LtfuListFilter
from .subject_screening_admin import SubjectScreeningAdmin


def get_part_two_fieldset_without_contact_number() -> Tuple[str, dict]:
    """Remove contact number from the part 2 fields"""
    part_two_name, part_two_dct = get_part_two_fieldset(collapse=True)
    part_two_dct = deepcopy(part_two_dct)
    part_two_dct["fields"] = [f for f in part_two_dct.get("fields") if f != "contact_number"]
    return part_two_name, part_two_dct


def get_fieldsets():
    return (
        get_part_one_fieldset(collapse=True),
        get_part_two_fieldset_without_contact_number(),
        get_part_three_report_datetime_fieldset(),
        get_part_three_vitals_fieldset(),
        get_part_three_pregnancy_fieldset(),
        get_part_three_glucose_fieldset(),
        get_part_three_repeat_glucose_fieldset(),
        get_part_three_creatinine_fieldset(),
        get_part_three_hba1c_fieldset(),
        comments_fieldset,
        calculated_values_fieldset,
        audit_fieldset_tuple,
    )


@admin.register(ScreeningPartThree, site=meta_screening_admin)
class ScreeningPartThreeAdmin(SiteModelAdminMixin, SubjectScreeningAdmin):
    form = ScreeningPartThreeForm

    fieldsets = get_fieldsets()

    list_filter = (
        # "part_three_report_datetime",
        "report_datetime",
        EligibilityPending,
        P3LtfuListFilter,
        P3ApptListFilter,
        "gender",
        "eligible",
        "eligible_part_one",
        "eligible_part_two",
        "eligible_part_three",
        "consented",
        "refused",
    )

    readonly_fields: Tuple[str, ...] = (
        *part_one_fields,
        *(f for f in part_two_fields if f != "contact_number"),
        *calculated_fields,
    )



from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NO
from edc_glucose.model_mixins import (
    fasting_model_mixin_factory,
    fbg_model_mixin_factory,
)
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class GlucoseFbg(
    CrfModelMixin,
    fasting_model_mixin_factory(
        None,
        fasting=models.CharField(
            verbose_name="Has the participant fasted?",
            max_length=15,
            choices=YES_NO,
            null=True,
            blank=False,
            help_text="As reported by patient",
        ),
    ),
    fbg_model_mixin_factory("fbg"),
    BaseUuidModel,
):

    fbg_performed = models.CharField(
        verbose_name="Was the FBG test performed?",
        max_length=15,
        choices=YES_NO,
    )

    fbg_not_performed_reason = models.CharField(
        verbose_name="If NO, provide reason",
        max_length=150,
        null=True,
        blank=True,
    )

    fasting_duration_estimated = models.CharField(
        max_length=15,
        default=NO,
        editable=False,
        help_text=(
            "Set to YES for existing values before duration "
            "question was added to the form, otherwise NO"
        ),
    )

    repeat_fbg_date = models.DateField(
        null=True,
        blank=True,
        help_text="Date should be within 1 week of report date",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Glucose (FBG)"
        verbose_name_plural = "Glucose (FBG)"



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext as _
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ...admin_site import meta_reports_admin
from ...models import MissingScreeningOgtt


@admin.register(MissingScreeningOgtt, site=meta_reports_admin)
class MissingScreeningOgttAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ["site", "screening_identifier"]
    list_display = [
        "dashboard",
        "screening_identifier",
        "site",
        "screening_datetime",
        "fbg_datetime",
        "fbg_value",
        "ogtt_value",
        "repeated",
        "p3_ltfu",
        "fbg2_value",
        "ogtt2_value",
        "fbg2_datetime",
        "ogtt2_datetime",
        "consented",
    ]

    list_filter = [
        ScheduleStatusListFilter,
        "screening_datetime",
        "fbg_datetime",
        "repeated",
        "p3_ltfu",
    ]

    search_fields = ["screening_identifier"]

    def dashboard(self, obj=None, label=None) -> str:
        url = self.get_subject_dashboard_url(obj=obj)
        if not url:
            url = reverse(
                "meta_screening_admin:meta_screening_subjectscreening_change",
                args=(obj.original_id,),
            )
        context = dict(title=_("Go to subject screening"), url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)



from copy import deepcopy
from datetime import datetime
from zoneinfo import ZoneInfo

import time_machine
from django.conf import settings
from django.contrib.sites.models import Site
from django.test import TestCase
from edc_constants.constants import COMPLETE, YES
from edc_metadata import KEYED, NOT_REQUIRED, REQUIRED
from edc_metadata.metadata import CrfMetadataGetter
from edc_metadata.metadata_handler import MetadataHandlerError
from edc_qol.constants import ALL_OF_THE_TIME, NONE_OF_THE_TIME, SOME_OF_THE_TIME
from edc_visit_schedule.constants import MONTH1, MONTH3, WEEK2

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_screening.tests.options import now
from meta_subject.forms import Sf12Form


@time_machine.travel(datetime(2019, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
class TestSf12(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_visit = self.get_subject_visit(appt_datetime=now)
        self.data = dict(
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
            general_health="excellent",
            moderate_activities_now_limited="limited_a_lot",
            climbing_stairs_now_limited="limited_a_lot",
            accomplished_less_physical_health=YES,
            work_limited_physical_health=YES,
            accomplished_less_emotional=YES,
            work_less_carefully_emotional=YES,
            pain_interfere_work="not_at_all",
            felt_calm_peaceful=ALL_OF_THE_TIME,
            felt_lot_energy=ALL_OF_THE_TIME,
            felt_down=NONE_OF_THE_TIME,
            social_activities_interfered=SOME_OF_THE_TIME,
            crf_status=COMPLETE,
            site=Site.objects.get(id=settings.SITE_ID),
        )

    def test_baseline_not_required(self):
        form = Sf12Form(data=self.data)
        form.is_valid()
        self.assertEqual({}, form._errors)
        self.assertRaises(MetadataHandlerError, form.save)

        crf_metadata_getter = CrfMetadataGetter(appointment=self.subject_visit.appointment)
        self.assertFalse(
            crf_metadata_getter.metadata_objects.filter(model="meta_subject.sf12").exists()
        )

    def test_1005_required(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        self.assertEqual(subject_visit.visit_code, WEEK2)
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(model="meta_subject.sf12").exists()
        )
        self.assertEqual(
            crf_metadata_getter.metadata_objects.get(
                model="meta_subject.sf12", visit_code=WEEK2
            ).entry_status,
            REQUIRED,
        )
        data = deepcopy(self.data)
        data.update(subject_visit=subject_visit, report_datetime=subject_visit.report_datetime)
        form = Sf12Form(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)
        form.save()
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=KEYED
            ).exists()
        )

    def test_1010_required_if_not_submitted(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(model="meta_subject.sf12").exists()
        )
        self.assertEqual(
            crf_metadata_getter.metadata_objects.get(
                model="meta_subject.sf12", visit_code=MONTH1
            ).entry_status,
            REQUIRED,
        )
        data = deepcopy(self.data)
        data.update(subject_visit=subject_visit, report_datetime=subject_visit.report_datetime)
        form = Sf12Form(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)
        form.save()
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=KEYED
            ).exists()
        )

    def test_1030_required_if_not_submitted(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        traveller = time_machine.travel(subject_visit.report_datetime)
        traveller.start()
        self.assertEqual(subject_visit.visit_code, MONTH3)
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(model="meta_subject.sf12").exists()
        )
        self.assertEqual(
            crf_metadata_getter.metadata_objects.get(
                model="meta_subject.sf12", visit_code=MONTH3
            ).entry_status,
            REQUIRED,
        )
        data = deepcopy(self.data)
        data.update(subject_visit=subject_visit, report_datetime=subject_visit.report_datetime)
        form = Sf12Form(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)
        form.save()
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=KEYED
            ).exists()
        )

    def test_1030_not_required_if_submitted(self):
        subject_visit_1005 = self.get_next_subject_visit(self.subject_visit)
        subject_visit_1010 = self.get_next_subject_visit(subject_visit_1005)
        subject_visit_1030 = self.get_next_subject_visit(subject_visit_1010)
        self.assertEqual(subject_visit_1030.visit_code, MONTH3)
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit_1030.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=REQUIRED
            ).exists()
        )
        data = deepcopy(self.data)
        data.update(
            subject_visit=subject_visit_1010,
            report_datetime=subject_visit_1010.report_datetime,
        )
        form = Sf12Form(data=data)
        form.is_valid()
        self.assertEqual({}, form._errors)
        form.save()
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit_1005.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=NOT_REQUIRED
            ).exists()
        )
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit_1010.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=KEYED
            ).exists()
        )
        crf_metadata_getter = CrfMetadataGetter(appointment=subject_visit_1030.appointment)
        self.assertTrue(
            crf_metadata_getter.metadata_objects.filter(
                model="meta_subject.sf12", entry_status=NOT_REQUIRED
            ).exists()
        )



mysql_view: str = """
    select *, uuid() as id, now() as created,
    'meta_reports.patienthistorymissingbaselinecd4' as report_model
      from (
        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,
          cd4_date, crf.site_id, crf.user_created, crf.user_modified,
          crf.modified
        from meta_subject_patienthistory as crf
        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id
        where cd4 is null or cd4_date is null
    ) as A
    order by subject_identifier
    """

pg_view: str = """
    select *, get_random_uuid() as id, now() as created,
    'meta_reports.patienthistorymissingbaselinecd4' as report_model
      from (
        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,
          cd4_date, crf.site_id, crf.user_created, crf.user_modified,
          crf.modified
        from meta_subject_patienthistory as crf
        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id
        where cd4 is null or cd4_date is null
    ) as A
    order by subject_identifier
"""

sqlite3_view = """
SELECT *, lower(
    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||
    substr(hex( randomblob(2)), 2) || '-' ||
    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||
    substr(hex(randomblob(2)), 2) || '-' ||
    hex(randomblob(6))
  ) as id, datetime() as `created`,
  'meta_reports.patienthistorymissingbaselinecd4' as report_model
      from (
        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,
          cd4_date, crf.site_id, crf.user_created, crf.user_modified,
          crf.modified
        from meta_subject_patienthistory as crf
        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id
        where cd4 is null or cd4_date is null
    ) as A
    order by subject_identifier
"""


def get_view_definition() -> dict:
    return {
        "django.db.backends.mysql": mysql_view,
        "django.db.backends.postgresql": pg_view,
        "django.db.backends.sqlite3": sqlite3_view,
    }



# Generated by Django 4.2.11 on 2024-07-27 00:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0007_alter_endpoints_endpoint_label_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endpoints",
            name="endpoint_label",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-16 00:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0199_auto_20240516_0247"),
    ]

    operations = [
        migrations.RenameField(
            model_name="bloodresultsins",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
        migrations.RenameField(
            model_name="glucosefbg",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
        migrations.RenameField(
            model_name="historicalbloodresultsins",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
        migrations.RenameField(
            model_name="historicalglucosefbg",
            old_name="fasting_duration_minutes",
            new_name="fasting_duration_delta",
        ),
    ]



from django.core.management import BaseCommand

from meta_analytics.dataframes import GlucoseEndpointsByDate


class Command(BaseCommand):

    def handle(self, *args, **options):
        print("Generating endpoints...")
        cls = GlucoseEndpointsByDate()
        cls.run()
        cls.to_model()
        print("Generating endpoints... done.")



from django.db import models
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class RepeatFbgRequest(CrfModelMixin, BaseUuidModel):
    """A model completed by the user to schedule a repeat FBG
    to determine if participant meets endpoint criteria.
    """

    next_fbg_date = models.DateField("Date particpant will attend to repeat FBG")

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass
